<?php

namespace Drupal\xray_audit\Plugin\tasks\PackageReport;

use Drupal\xray_audit\Plugin\XrayAuditTaskPluginBase;
use Symfony\Component\DependencyInjection\ContainerInterface;
use Drupal\Core\Config\StorageInterface;
use Drupal\Core\Extension\ModuleExtensionList;

/**
 * Plugin implementation of modules status.
 * @XrayAuditTaskPlugin (
 *   id = "modules",
 *   label = @Translation("Modules Status (sync config files)"),
 *   description = @Translation("Module status against sync configuration files (include config splits)."),
 *   group = "package",
 *   sort = 1,
 *   operations = {
 *      "modules" = {
 *          "label" = "Drupal Modules",
 *          "description" = "Drupal Modules."
 *       }
 *    },
 * )
 */
class XrayAuditModulesPlugin extends XrayAuditTaskPluginBase {

  /**
   * Service "config.storage.sync".
   *
   * @var \Drupal\Core\Config\StorageInterface
   */
  protected $configStoreSync;

  /**
   * Service "extension.list.module".
   *
   * @var \Drupal\Core\Extension\ModuleExtensionList
   */
  protected $extensionListModule;

  /**
   * Constructs a \Drupal\Component\Plugin\PluginBase object.
   *
   * @param array $configuration
   *   A configuration array containing information about the plugin instance.
   * @param string $plugin_id
   *   The plugin_id for the plugin instance.
   * @param mixed $plugin_definition
   *   The plugin implementation definition.
   * @param \Drupal\Core\Config\StorageInterface $configStoreSync
   *   Service "config.storage.sync".
   * @param \Drupal\Core\Extension\ModuleExtensionList $extensionListModule
   *   Service "extension.list.module".
   */
  public function __construct(array $configuration, string $plugin_id, $plugin_definition, StorageInterface $configStoreSync, ModuleExtensionList $extensionListModule) {
    parent::__construct($configuration, $plugin_id, $plugin_definition);
    $this->configStoreSync = $configStoreSync;
    $this->extensionListModule = $extensionListModule;
  }

  /**
   * {@inheritdoc}
   */
  public static function create(ContainerInterface $container, array $configuration, $plugin_id, $plugin_definition) {
    return new static(
      $configuration,
      $plugin_id,
      $plugin_definition,
      $container->get('config.storage.sync'),
      $container->get('extension.list.module'),
    );
  }

  /**
   * {@inheritdoc}
   */
  public function getDataOperationResult(string $operation = NULL) {
    switch ($operation) {

      case 'modules':
        return $this->getModuleStatus();

    }
    return [];

  }

  /**
   * Modules custom and contrib.
   *
   * @param int $core
   *   Modules core or contrib and custom.
   * @return array
   *   Data.
   */
  protected function getModuleStatus() {
    $resultTable = [];
    $list_modules_instaled = $this->extensionListModule->getList();
    $modules_actives =  $this->getModulesActivesByConfigSync();
    if (empty($list_modules_instaled) || empty($modules_actives)) {
      return $resultTable;
    }

    // Build the data.
    foreach($list_modules_instaled as $machine_name => $module) {
        $group = $module->origin === 'core' ? 'core' : 'not_core';
        $info = $module->info;
        $resultTable[$group][$machine_name] = [
          'package' => $info['package'] ?? '',
          'project' => $info['project'] ?? '',
          'module' => $info['name'],
          'machine_name' => "$machine_name",
          'used' => isset($modules_actives[$machine_name]) ? $this->t('Yes') : $this->t('No'),
          'enabled_by' => $modules_actives[$machine_name] ?? '',
        ];
    }

    return $resultTable;

  }

  /**
   * {@inheritDoc}
   */
  public function buildDataRenderArray(array $data, string $operation = NULL) {
    if (empty($data)) {
      return [
        "#markup" => $this->t("Not data found, 
              maybe the configuration files have not been exported.")
      ];
    }

    $headerTable = [
      $this->t('Package'),
      $this->t('Project'),
      $this->t('Module'),
      $this->t('Machine name'),
      $this->t('Used'),
      $this->t('Activeted by'),
    ];

    $group_info['core'] = [
      'title' => $this->t("Modules core"),
      'description' => $this->t("Modules belong to the core"),
    ];
    $group_info['not_core'] = [
      'title' => $this->t("Modules custom and contrib"),
      'description' => $this->t("Modules custom and contrib"),
    ];

    $builds = [];
    foreach($data as $type => $group_modules) {
      // Sort the modules by package.
      $key_values = array_column($group_modules, 'package');
      array_multisort($key_values, SORT_ASC, $group_modules);

      $builds[$type] = [
        '#theme' => 'details',
        '#title' => $group_info[$type]['title'],
        '#description' => $group_info[$type]['description'],
        '#attributes' => ['class' => ['package-listing']],
        '#children' => [
          'tables' => [
            '#theme' => 'table',
            '#header' => $headerTable,
            '#rows' => $group_modules,
          ]
        ]
      ];
    }

    return $builds;

  }

  /**
   * Get the modules actives by configuration files.
   *
   * @return array
   *    Status modules.
   */
  protected function getModulesActivesByConfigSync() :  array {
    // Set the module actives checking the configuration files.
    $modules_actives = [];
    // Get the name of all the config splits configuration files.
    $all_config_splits = $this->configStoreSync->listAll('config_split.config_split.');

    // Add the core.extension (modules enable by default).
    $all_config_splits[] = 'core.extension';

    // Storage the active modules.
    foreach($all_config_splits as $config_split_name) {
      $config_split = $this->configStoreSync->read($config_split_name);
      if (empty($config_split['module'])) {
        continue;
      }
      foreach ($config_split['module'] as $name => $value) {
        if (!isset($modules_actives[$name])) {
          $modules_actives[$name] = '';
        } else {
          $modules_actives[$name] .= ', ';
        }
        $modules_actives[$name] .= $config_split_name;
      }
    }

    return $modules_actives;

  }

}
