<?php

namespace Drupal\xray_audit\Plugin\tasks\QueryGroup;

use Drupal\user\Entity\Role;

/**
 * Plugin implementation of queries_data_node.
 *
 * @XrayAuditTaskPlugin (
 *   id = "queries_data_user",
 *   label = @Translation("Data about User and Roles"),
 *   description = @Translation("Queries execute on database to get reports about user."),
 *   group = "queries_data",
 *   sort = 3,
 *   operations = {
 *      "count_user" = {
 *          "label" = "User count",
 *          "description" = "Number of user."
 *       },
 *      "user_less_activity" = {
 *          "label" = "User activity",
 *          "description" = "Count the last access to the site by users."
 *       },
 *       "description_of_roles" = {
 *          "label" = "Roles",
 *          "description" = "Roles"
 *       },
 *      "count_user_per_role" = {
 *          "label" = "User count per roles",
 *          "description" = "User count per roles."
 *       },
 *       "permissions_roles" = {
 *          "label" = "Permissions per role",
 *          "description" = "Permissions per role."
 *       }
 *    },
 *   dependencies = {"user"}
 * )
 */
class XrayAuditQueryTaskUserPlugin extends XrayAuditQueryTaskPluginBase {

  /**
   * {@inheritdoc}
   */
  public function getDataOperationResult(string $operation = NULL) {
    switch ($operation) {
      case 'count_user':
        return $this->countUser();

      case 'user_less_activity':
        return $this->userLessActivity();

      case 'count_user_per_role':
        return $this->usersPerRole();

      case 'description_of_roles':
        return $this->rolesDescription();

      case 'permissions_roles':
        return $this->permissionsPerRole();
    }
    return [];
  }

  /**
   * Get data for operation "count_user".
   *
   * @return array
   *   Data.
   */
  protected function countUser() {

    $headerTable = [$this->t('Status'), $this->t('Number of users')];

    $aggregateQuery = $this->entityTypeManager->getStorage('user')->getAggregateQuery();

    $resultTable = $aggregateQuery->accessCheck(FALSE)
      ->aggregate('uid', 'count')
      ->condition('uid', 0, '!=')
      ->groupBy('status')
      ->execute();

    $total = 0;
    foreach ($resultTable as &$row) {
      $row['status'] = $row['status'] == 1 ? $this->t('Active') : $this->t('Blocked');
      $total += $row['uid_count'];
    }

    $resultTable[] = [
      'status' => $this->t('Total'),
      'user_count' => $total,
    ];

    return [
      'header_table' => $headerTable,
      'results_table' => $resultTable,
    ];

  }

  /**
   * Activity users.
   *
   * @return array
   *   Data.
   */
  protected function userLessActivity() {
    $headerTable = [$this->t('Last access'), $this->t('Number of users')];
    $userStorage = $this->entityTypeManager->getStorage('user');
    $current_time = time();

    $one_month_seconds = 2592000;

    // Users last access in 6 months.
    $user_count_6_m = $userStorage->getQuery()
      ->accessCheck(FALSE)
      ->condition('status', 1)
      ->condition('access', $current_time - ($one_month_seconds * 6), '>')
      ->count()
      ->execute();

    // Users lasd access 6 to 12 months.
    $user_count_6_12_m = $userStorage->getQuery()
      ->accessCheck(FALSE)
      ->condition('status', 1)
      ->condition('access', $current_time - ($one_month_seconds * 6), '<')
      ->condition('access', $current_time - ($one_month_seconds * 12), '>')
      ->count()
      ->execute();

    // Users access more than 12 months.
    $user_count_more_12_m = $userStorage->getQuery()
      ->accessCheck(FALSE)
      ->condition('status', 1)
      ->condition('access', $current_time - ($one_month_seconds * 12), '<')
      ->count()
      ->execute();

    $resultTable = [
      [$this->t('Last 6 months'), $user_count_6_m],
      [$this->t('Last 6 to 12 months'), $user_count_6_12_m],
      [$this->t('More than 12 months'), $user_count_more_12_m],
    ];

    return [
      'header_table' => $headerTable,
      'results_table' => $resultTable,
    ];

  }

  /**
   * Users per role.
   *
   * @return array
   *   Render array.
   */
  public function usersPerRole() {

    $headerTable = [$this->t('roles'), $this->t('count')];
    $resultTable = [];

    $query = $this->database->query("select roles_target_id as role, count(1) as count from user__roles group by role");
    $result = $query->fetchAll();

    foreach ($result as $row) {
      $resultTable[] = [$row->role, $row->count];
    }

    return [
      'header_table' => $headerTable,
      'results_table' => $resultTable,
    ];

  }

  /**
   * Description roles.
   *
   * @return array
   *   Render array.
   */
  public function rolesDescription() {
    $headerTable = [
      $this->t('id'),
      $this->t('label'),
      $this->t('langcode'),
    ];
    $resultTable = [];
    $roles = Role::loadMultiple();
    foreach ($roles as $role) {
      $resultTable[] = [
        $role->get('id'),
        $role->get('label'),
        $role->get('langcode'),
      ];
    }
    return [
      'header_table' => $headerTable,
      'results_table' => $resultTable,
    ];
  }

  /**
   * Permissions and roles.
   *
   * @return array
   *   Render array.
   */
  public function permissionsPerRole() {
    $headerTable = [
      $this->t('role'),
      $this->t('permissions'),
    ];
    $resultTable = [];
    $roles = Role::loadMultiple();
    foreach ($roles as $role) {
      $resultTable[] = [
        $role->get('label'),
        implode(', ', $role->get('permissions')),
      ];
    }
    return [
      'header_table' => $headerTable,
      'results_table' => $resultTable,
    ];
  }

}
