<?php

namespace Drupal\xray_audit\Services;

use Drupal\Component\Render\FormattableMarkup;
use Drupal\Component\Utility\Html;
use Drupal\Core\Entity\Display\EntityDisplayInterface;
use Drupal\Core\Entity\EntityFieldManagerInterface;
use Drupal\Core\Entity\EntityTypeManagerInterface;
use Drupal\Core\Field\FieldDefinitionInterface;

/**
 * Retrieve data from entity about display modes.
 */
class DisplayModesEntityArchitecture implements DisplayModesEntityArchitectureInterface {

  /**
   * The entity type manager.
   *
   * @var \Drupal\Core\Entity\EntityTypeManagerInterface
   */
  protected $entityTypeManager;

  /**
   * The entity file manager.
   *
   * @var \Drupal\Core\Entity\EntityFieldManagerInterface
   */
  protected $entityFieldManager;

  /**
   * Headers for table.
   *
   * @var string[]
   */
  protected $headers = ['type' => 'Type', 'fields' => 'Fields'];

  /**
   * Headers for table (display views)
   *
   * @var array
   */
  protected $headersDisplayDiews = [];

  /**
   * Config, excluded fields.
   *
   * @var array
   */
  protected $excludedFields = [];

  /**
   * Storage computed fields.
   *
   * @var array
   */
  protected $computed = [];

  /**
   * Construct service Extractor Display Modes.
   *
   * @param \Drupal\Core\Entity\EntityTypeManagerInterface $entityTypeManager
   *   Service Entity Type Manager.
   * @param \Drupal\Core\Entity\EntityFieldManagerInterface $entityFieldManager
   *   Service Entity Field Manager.
   */
  public function __construct(EntityTypeManagerInterface $entityTypeManager, EntityFieldManagerInterface $entityFieldManager) {
    $this->entityTypeManager = $entityTypeManager;
    $this->entityFieldManager = $entityFieldManager;
  }

  /**
   * {@inheritdoc}
   */
  public function getData(string $entity_type, string $entity) {
    $this->computed = [];
    $results_extracted = [];
    $results_table = [];

    $type_entities = $this->entityTypeManager->getStorage($entity_type)->loadMultiple();

    foreach ($type_entities as $type_entity) {
      $results_extracted[] = [
        'bundle' => $type_entity->label(),
        'field_data' => $this->getFieldDataFromEntity($entity, $type_entity->id()),
        'display_view' => $this->getViewDisplayData($entity, $type_entity->id()),
      ];
    }

    $array_structure = [];
    foreach (array_merge($this->headers, $this->headersDisplayDiews) as $key => $row) {
      $array_structure[$key] = '';
    }

    foreach ($results_extracted as $results_extracted_item) {
      $this->processDataForTable($results_extracted_item, $array_structure, $results_table);
    }

    return [
      'computed' => !empty($this->computed) ? implode(', ', $this->computed) : NULL,
      'header_table' => array_merge($this->headers, $this->headersDisplayDiews),
      'results_table' => $results_table,
    ];

  }

  /**
   * Extract info from configuration about entity Fields.
   *
   * @param string $entity
   *   Entity machine, for example, node.
   * @param string $type
   *   Machine name of Type.
   *
   * @return array
   *   Data.
   */
  protected function getFieldDataFromEntity(string $entity, string $type) {
    $field_data = [];
    $fields = $this->entityFieldManager->getFieldDefinitions($entity, $type);
    if (empty($fields)) {
      return [];
    }

    usort($fields, function ($a, $b) {
      return strnatcmp(($a->getLabel() ?? $a->getName()), ($b->getLabel() ?? $b->getName()));
    });

    foreach ($fields as $field) {

      $name = $field->getName() ?? '';
      // Check that the fields can be showed in the front.
      $display_option_view = $field->getDisplayOptions('view');
      $display_option_view = $display_option_view['region'] ?? '';
      if (in_array($name, $this->excludedFields)
        || ($field->isDisplayConfigurable('view') === FALSE && ($field->getDisplayOptions('view') === NULL || $display_option_view === 'hidden'))) {
        continue;
      }

      $type = $field->getType() ?? '';
      $label = $field->getLabel() ?? $name;

      $settings = '';
      if (in_array($type, ['entity_reference', 'entity_reference_revisions'])) {
        $settings = $this->preparedSettingsEntityReference($field);
      }
      elseif ($type === 'text_long') {
        $settings = $this->preparedSettingsTextLong($field);
      }

      $field_data[$name] = [
        'name' => $name,
        'label' => (string) $label,
        'type' => $type,
        'settings' => $settings,
      ];
    }

    return $field_data;
  }

  /**
   * Preprocess settings of a field Entity Reference.
   *
   * @param \Drupal\Core\Field\FieldDefinitionInterface $field
   *   Field.
   *
   * @return string
   *   Settings.
   */
  protected function preparedSettingsEntityReference(FieldDefinitionInterface $field) {
    $settings = $field->getSetting('target_type');
    $handler_settings = $field->getSetting('handler_settings')['target_bundles'] ?? '';
    if (is_array($handler_settings)) {
      $settings .= ': ' . $this->implodeArrayKeyValue($handler_settings, FALSE);
    }

    return $settings;
  }

  /**
   * Custom implode array.
   *
   * @param array $array
   *   Array.
   * @param bool $with_key
   *   Flag if we want keys in the result.
   *
   * @return string
   *   Imploded array.
   */
  protected function implodeArrayKeyValue(array $array, bool $with_key = TRUE) {
    $imploded = '';
    foreach ($array as $key => $element) {
      if (empty($element) || is_array($element)) {
        continue;
      }
      if ($imploded !== '') {
        $imploded .= ' | ';
      }
      if ($with_key) {
        $imploded .= "$key : ";
      }
      if ($key === 'plugin_types') {
        $this->processPlugintype($element);
      }
      if ($key === 'trim_length') {
        $imploded .= "$element";
        break;
      }
      if ($key === 'image_loading') {
        return "";
      }
      $imploded .= "$element";
    }

    return $imploded;

  }

  /**
   * Process plugin type.
   *
   * @param array $element
   *   Plugin type element.
   */
  protected function processPlugintype(array &$element) {
    foreach ($element as $element_value) {
      if (is_string($element_value)) {
        $element = $element_value;
      }
    }
  }

  /**
   * Prepare settings of Text Long Field.
   *
   * @param \Drupal\Core\Field\FieldDefinitionInterface $field
   *   Field.
   *
   * @return string
   *   Settings.
   */
  protected function preparedSettingsTextLong(FieldDefinitionInterface $field) {
    $allowed_formats = $field->getSetting('allowed_formats');
    if (empty($allowed_formats)) {
      return '';
    }
    $settings = 'Formats =>';
    foreach ($allowed_formats as $allowed_format) {
      if ($allowed_format == 0) {
        continue;
      }

      $settings .= " $allowed_format";
    }

    return $settings;
  }

  /**
   * Extract info from configuration about displays.
   *
   * @param string $entity
   *   Entity machine, for example, node.
   * @param string $bundle
   *   Machine name of Type.
   *
   * @return array
   *   Data.
   */
  protected function getViewDisplayData(string $entity, string $bundle) {
    $component_configurations = ['default' => []];
    $this->headersDisplayDiews['default'] = 'default';
    $display_modes = $this->getEntityViewDisplay($entity, $bundle);

    foreach ($display_modes as $display_mode) {
      $mode = $display_mode->getMode();
      $this->headersDisplayDiews[$mode] = $mode;
      $component_configurations[$mode] = $this->getDataFromOneDisplayMode($display_mode);
    }

    return $component_configurations;
  }

  /**
   * Get Display views of an entity.
   *
   * @param string $entity
   *   Entity machine name.
   * @param string $bundle
   *   Bundle machine name.
   *
   * @return \Drupal\Core\Entity\EntityInterface[]
   *   Display views.
   */
  protected function getEntityViewDisplay(string $entity, string $bundle) {
    $entity_display_view = $this->entityTypeManager->getStorage('entity_view_display');
    $display_view = $entity_display_view->loadByProperties([
      'targetEntityType' => $entity,
      'bundle' => $bundle,
    ]);

    // If there are not display modes we set the default one.
    if (empty($display_view)) {
      $values = [
        'targetEntityType' => $entity,
        'bundle' => $bundle,
        'mode' => 'default',
        'status' => TRUE,
      ];
      $default_display_mode = $entity_display_view->create($values);
      $default_display_mode->save();
      $display_view = $entity_display_view->loadByProperties([
        'targetEntityType' => $entity,
        'bundle' => $bundle,
      ]);
    }

    return $display_view;
  }

  /**
   * Get the display mode configuration.
   *
   * @param \Drupal\Core\Entity\Display\EntityDisplayInterface $display_mode
   *   Display mode.
   *
   * @return array
   *   Fields of display mode.
   */
  protected function getDataFromOneDisplayMode(EntityDisplayInterface $display_mode): array {
    $component_configurations = [];
    $components = $display_mode->getComponents();
    foreach ($components as $key => &$component_alpha) {
      $component_alpha['machinename'] = $key;
    }
    usort($components, function ($a, $b) {
      return $a['weight'] <=> $b['weight'];
    });
    foreach ($components as $component) {
      $machinename = $component['machinename'];
      if (in_array($machinename, $this->excludedFields)) {
        continue;
      }
      $component_configurations[$machinename] = [
        'type' => $component['type'] ?? NULL,
        'label' => $component['label'] ?? NULL,
        'settings' => !empty($component['settings']) ? $this->implodeArrayKeyValue($component['settings']) : '',
      ];
    }

    return $component_configurations;
  }

  /**
   * Prepare the data for theme table.
   *
   * @param array $results_extracted
   *   Data extract from configuration.
   * @param array $array_structure
   *   Table structure.
   * @param array $results
   *   Processed values.
   */
  protected function processDataForTable(array $results_extracted, array $array_structure, array &$results) {
    $result = $array_structure;

    $result['type'] = new FormattableMarkup("<p>@bundle</p>", ['@bundle' => $results_extracted['bundle']]);
    $bundle_sanitized = Html::cleanCssIdentifier(strtolower($results_extracted['bundle']));

    $fields = '';
    $place_holders = [];
    $count = 0;
    foreach ($results_extracted['field_data'] as $field) {
      $type_settings = '';

      if (!empty($field['type']) && !empty($field['settings'])) {
        $type_settings = "{$field['type']} => {$field['settings']}";
      }
      else {
        $type_settings = "{$field['type']} {$field['settings']}";
      }

      $type_settings = trim($type_settings);

      $bundle_sanitized_id = "@bundle_sanitized_" . $count;
      $field_name_id = "@field_name_" . $count;
      $field_label_id = "@field_label_" . $count;
      $type_settings_id = "@type_settings_" . $count;

      $fields .= "<p data-highlight-target=\"{$bundle_sanitized_id}__{$field_name_id}\" class=\"xray-audit__field {$bundle_sanitized_id}__{$field_name_id}\"><b>{$field_label_id} ({$field_name_id})</b> ({$type_settings_id})</p>";

      $place_holders[$bundle_sanitized_id] = $bundle_sanitized;
      $place_holders[$field_name_id] = $field['name'];
      $place_holders[$field_label_id] = $field['label'];
      $place_holders[$type_settings_id] = $type_settings;

      $count ++;
    }

    $result['fields'] = new FormattableMarkup($fields, $place_holders);

    foreach ($results_extracted['display_view'] as $mode => $display_data) {
      if (is_array($display_data)) {
        $result[$mode] = $this->processDisplayDataField($results_extracted, $display_data, $bundle_sanitized);
        continue;
      }
      $result[$mode] = new FormattableMarkup('@display_data', ['@display_data' => $display_data]);

    }


    $results[] = $result;
  }

  /**
   * Process data field.
   *
   * @param array $results_extracted
   *   Results estracted.
   * @param array $display_data
   *   Display data.
   * @param string $bundle_sanitized
   *   Sanitized bundle.
   *
   */
  protected function processDisplayDataField(array $results_extracted, array $display_data, string $bundle_sanitized) {
    $mode_display_data_conf = '';
    $place_holders = [];
    $count = 0;
    foreach ($display_data as $machiname => $display_data_field) {
      if (empty($results_extracted['field_data'][$machiname]['label'])) {
        $label = $machiname;
        $type = 'computed';
        $settings = '';
        $this->computed[$machiname] = $machiname;
      }
      else {
        $label = $results_extracted['field_data'][$machiname]['label'];
        $type = $results_extracted['field_data'][$machiname]['type'] ?? '';
        $settings = '';
        if ($display_data_field['type'] !== '') {
          $settings = $display_data_field['type'] . ' ';
        }
        if (!empty($display_data_field['settings'])) {
          $settings .= $display_data_field['settings'];
        }
        $settings = trim($settings);
      }

      $type_settings = '';
      if (!empty($type) && !empty($settings)) {
        $type_settings = "{$type} => {$settings}";
      }
      else {
        $type_settings = "{$type} {$settings}";
      }

      $type_settings = trim($type_settings);

      $bundle_sanitized_id = "@bundle_sanitized_" . $count;
      $machiname_id = "@machiname_" . $count;
      $label_id = "@label_" . $count;
      $type_settings_id = "@type_settings_" . $count;

      $mode_display_data_conf .= "<p data-highlight-target=\"{$bundle_sanitized_id}__{$machiname_id}\" class=\"xray-audit__field {$bundle_sanitized_id}__{$machiname_id}\"><b>{$label_id}</b>({$type_settings_id})</p>";

      $place_holders[$bundle_sanitized_id] = $bundle_sanitized;
      $place_holders[$machiname_id] = $machiname;
      $place_holders[$label_id] = $label;
      $place_holders[$type_settings_id] = $type_settings;

      $count ++;
    }

    return new FormattableMarkup($mode_display_data_conf, $place_holders);

  }

}
